 /**
弹出框组件
 @class ui.Popup
*/
;ui.Popup = (function($){

    var _popup,_mask,transition,clickMask2close,
        POSITION = {
            'top':{
                top:0,
                left:0,
                right:0
            },
            'top-second':{
                top:'70px',
                left:0,
                right:0
            },
            'center':{
                top:'50%',
                left:'5%',
                right:'5%',
                'border-radius' : '3px'
            },
            'bottom' : {
                bottom:0,
                left:0,
                right:0
            },
            'bottom-second':{
                bottom : '72px',
                left:0,
                right:0
            }
        },
        ANIM = {
            top : ['slideDownIn','slideUpOut'],
            bottom : ['slideUpIn','slideDownOut'],
            defaultAnim : ['bounceIn','bounceOut']
        },
        TEMPLATE = {
            alert : '<div class="popup-title">{title}</div><div class="popup-content">{content}</div><div id="popup_btn_container"><a data-target="closePopup" data-icon="checkmark">{ok}</a></div>',
            confirm : '<div class="popup-title">{title}</div><div class="popup-content">{content}</div><div id="popup_btn_container"><a class="cancel" data-icon="close">{cancel}</a><a data-icon="checkmark">{ok}</a></div>',
            loading : '<i class="icon {icon}"></i><p>{title}</p>',
            loadingOnlyIcon : '<i class="icon iconOnly {icon}"></i>'
        };

    /*
     * 全局只有一个popup实例
     * @private
     */
    var _init = function(){
        $('body').append('<div id="bingotouch_popup"></div><div id="bingotouch_popup_mask"></div>');
        _mask = $('#bingotouch_popup_mask');
        _popup = $('#bingotouch_popup');
        _subscribeEvents();
    }

    /**
        打开窗口
        @method  ui.Popup.show(options)
        @param options {JSONObject} (可选) 参数，具体参数如下
        <ul>
        <li>height (Number):  高度</li>
        <li>width (Number):  宽度 </li>
        <li>opacity (Number):  遮罩层透明度 | 默认值 0.3</li>
        <li>html (String):  popup内容 </li>
        <li>pos (String):  位置  | 取值 top|top-second|center|bottom|bottom-second </li>
        <li>clickMask2Close (Boolean):  是否点击外层遮罩关闭popup | 默认值 true </li>
        <li>showCloseBtn (Boolean):  是否显示关闭按钮 | 默认值 true </li>
        <li>animation (Boolean):  是否显示动画 | 默认值 true </li>
        <li>duration (Number):  动画执行时间 | 默认值 200ms </li>     
        </ul>
        @example
            ui.Popup.show({
                html: '<div style="height: 100px;text-align: center;font-size: 20px;font-weight: 600;margin-top: 10px;color:#E74C3C ">随意设计你的弹出框吧</div>',
                pos : 'center'
            })
    */
    var show = function(options){
        var settings = {
            height : undefined,//高度
            width : undefined,//宽度
            opacity : 0.3,//透明度
            url : null,//远程加载url
            tplId : null,//加载模板ID
            tplData : null,//模板数据，配合tplId使用
            html : '',//popup内容
            pos : 'center',//位置 {@String top|top-second|center|bottom|bottom-second}   {@object  css样式}
            clickMask2Close : true,// 是否点击外层遮罩关闭popup
            showCloseBtn : true,// 是否显示关闭按钮
            arrowDirection : undefined,//popover的箭头指向
            animation : true,//是否显示动画
            timingFunc : 'linear',
            duration : 200,//动画执行时间
            onShow : undefined //@event 在popup内容加载完毕，动画开始前触发
        }
        $.extend(settings,options);
        clickMask2close = settings.clickMask2Close;
        _mask.css('opacity',settings.opacity);
        //rest position and class
        _popup.attr({'style':'','class':''});
        settings.width && _popup.width(settings.width);
        settings.height && _popup.height(settings.height);
        var pos_type = $.type(settings.pos);
        if(pos_type == 'object'){// style
            _popup.css(settings.pos);
            transition = ANIM['defaultAnim'];
        }else if(pos_type == 'string'){
            if(POSITION[settings.pos]){ //已经默认的样式
                _popup.css(POSITION[settings.pos])
                var trans_key = settings.pos.indexOf('top')>-1?'top':(settings.pos.indexOf('bottom')>-1?'bottom':'defaultAnim');
                transition = ANIM[trans_key];
            }else{// pos 为 class
                _popup.addClass(settings.pos);
                transition = ANIM['defaultAnim'];
            }
        }else{
            console.error('错误的参数！');
            return;
        }
        _mask.show();
        var html;
        if(settings.html){
            html = settings.html;
        }else if(settings.url){//远程加载
            //html = J.Page.loadContent(settings.url);
        }else if(settings.tplId){//加载模板
            //html = template(settings.tplId,settings.tplData)
        }

        //是否显示关闭按钮
        if(settings.showCloseBtn){
            html += '<div id="tag_close_popup" data-target="closePopup" class="icon icon-cancel"></div>';
        }
        //popover 箭头方向
        if(settings.arrowDirection){
            _popup.addClass('arrow '+settings.arrowDirection);
            _popup.css('padding','8px');
            if(settings.arrowDirection=='top'||settings.arrowDirection=='bottom'){
                transition = ANIM[settings.arrowDirection];
            }
        }

        _popup.html(html).show();

        //J.Element.init(_popup); 
        //执行onShow事件，可以动态添加内容
        settings.onShow && settings.onShow.call(_popup);

        //显示获取容器高度，调整至垂直居中
        if(settings.pos == 'center'){
            var height = _popup.height();
            _popup.css('margin-top','-'+height/2+'px')
        }
        if(settings.animation){
            ui.anim(_popup,transition[0],settings.duration,settings.timingFunc);
        }
        ui.hasPopupOpen = true;
    }

    /**
       关闭弹出框
        @method  ui.Popup.hide(noTransition)
        @param noTransition {Boolean}  是否立即关闭，无动画  | 默认值是false
        @example
            ui.Popup.hide();
    */
    var hide = function(noTransition){
        _mask.hide();
        if(transition && !noTransition){
            ui.anim(_popup,transition[1],200,function(){
                _popup.hide().empty();
                ui.hasPopupOpen = false;
            });
        }else{
            _popup.hide().empty();
            ui.hasPopupOpen = false;
        }

    }
    var _subscribeEvents = function(){
        _mask.on('tap',function(){
            clickMask2close &&  hide();
        }).on("touchmove",function(e){
            e.preventDefault();
        });
        _popup.on('tap','[data-target="closePopup"]',function(){
            hide();
        }).on("touchmove",function(e){
            e.preventDefault();
        });
    }

    /**
        alert组件
        @method  ui.Popup.alert(title,content,btnName)
        @param title {String} 标题
        @param content {String}  内容
        @param btnName {String}  按钮名称 | 默认值 '确定'
        @example
            ui.Popup.alert('提示','这是一个Alert');
    */    
    var alert = function(title,content,btnName){
        var markup = TEMPLATE.alert.replace('{title}',title).replace('{content}',content).replace('{ok}',btnName || '确定');
        show({
            html : markup,
            pos : 'center',
            clickMask2Close : false,
            showCloseBtn : false
        });
    }

    /**
        confirm 组件
        @method  ui.Popup.confirm(title,content,okCall,cancelCall,okBtnName,cancelBtnName)
        @param title {String}  标题
        @param content {String}  内容
        @param okCall {Function} 确定按钮handler
        @param cancelCall {Function} 取消按钮handler
        @param okBtnName {String}  （可选）确认按钮名称 | 默认值 '确定'
        @param cancelBtnName {String}  （可选）取消按钮名称 | 默认值 '取消'
        @example
            ui.Popup.confirm('提示','这是一个Confirm！',function(){app.hint('你选择了“确定”')},function(){app.hint('你选择了“取消”')});
    */     
    var confirm = function(title,content,okCall,cancelCall, okBtnName, cancelBtnName){
        var markup = TEMPLATE.confirm.replace('{title}',title).replace('{content}',content).replace('{cancel}', cancelBtnName || '取消').replace('{ok}', okBtnName || '确定');
        show({
            html : markup,
            pos : 'center',
            clickMask2Close : false,
            showCloseBtn : false
        });
        $('#popup_btn_container [data-icon="checkmark"]').tap(function(){
            hide();
            okCall.call(this);
        });
        $('#popup_btn_container [data-icon="close"]').tap(function(){
            hide();
            cancelCall.call(this);
        });
    }

    /**
        带箭头的弹出框
        @method  ui.Popup.popover(html,pos,arrow_direction,onShow)
        @param html {String} 弹出框内容
        @param pos {String}  位置  | 取值 top|top-second|center|bottom|bottom-second
        @param arrow_direction {String}  箭头方向 | 取值 top|bottom
        @param onShow  {Function} 显示之前执行
        @example
            var html = '<ul id="popoverlist" class="list-view"> ' +
                            '<li><div data-role="BTButton">Hello, BingoTouch</div></li>' +
                            '<li><div data-role="BTButton">Good good Study</div></li>' +
                            '<li><div data-role="BTButton">Day day up</div></li>' +
                        '</ul>';    
            ui.Popup.popover(html,{
                top:'70px',
                left:'10%',
                right:'10%'
            },'top');

            $("#popoverlist").uiwidget();
    */     
    var popover = function(html,pos,arrow_direction,onShow){
        show({
            html : html,
            pos : pos,
            showCloseBtn : false,
            arrowDirection : arrow_direction,
            onShow : onShow
        });
    }

    /*
        loading组件
        @method  ui.Popup.loading(text, icon)
        @param text {String} 文本 |  当为null时，不显示文字
        @param icon {String}  加载中字体图标  | 默认值 'icon-spinner5'
        @example
            ui.Popup.loading("加载中...");
    */     
    var loading = function(text, icon){
        var markup;
        if(text == null){
            markup = TEMPLATE.loadingOnlyIcon.replace('{icon}', icon||'icon-spinner5');      
        } else{
            markup = TEMPLATE.loading.replace('{title}', text).replace('{icon}', icon||'icon-spinner5');     
        }
         show({
            html : markup,
            pos : 'loading',
            opacity : 0,
            duration : 10,
            clickMask2Close : false
        });
    }

    
    /**
        actionsheet组件
        @method   ui.Popup.actionsheet(buttons)
        @param buttons {Array}  按钮集合
        [{color:'red',text:'btn',handler:function(){}},{color:'red',text:'btn',handler:function(){}}]
        @example
            ui.Popup.actionsheet([{
                color : "#27AE60",
                text : '告诉QQ好友',
                handler : function(){
                    app.hint('告诉QQ好友！');
                }
            },{
                color : "#27AE60",
                text : '告诉MSN好友',
                handler : function(){
                    app.hint('告诉MSN好友！');
                }
            }
            ]);
    */     
    var actionsheet = function(buttons){
        var markup = '<div class="actionsheet">';
        $.each(buttons,function(i,n){
            //markup += '<button style="background-color: '+ n.color +' !important;">'+ n.text +'</button>';

            markup += '<div data-role="BTButton" style="background-color:'+ n.color +'" data-status="1"><span class="btn-text">'+ n.text +'</span></div>';
        });
        markup += '<div data-role="BTButton" data-theme="d" data-status="1"><span class="btn-text">取消</span></div>';
        markup += '</div>';
        show({
            html : markup,
            pos : 'bottom',
            showCloseBtn : false,
            onShow : function(){
                $(this).find('[data-role="BTButton"]').each(function(i,button){
                    $(button).on('tap',function(){
                        if(buttons[i] && buttons[i].handler){
                            buttons[i].handler.call(button);
                        }
                        hide();
                    });
                });
            }
        });
    }

    $(function(){
        _init();
    });


    return {
        show : show,
        close : hide,
        alert : alert,
        confirm : confirm,
        popover : popover,
        loading : loading,
        actionsheet : actionsheet
    }
})(Zepto);