/**
换肤
 @class ui.Skin
*/
/**
    换肤初始化（header和footer默认添加到bgColorSelector）
    @method  ui.Skin.init(options)
    @param color {String}  字体颜色 | 默认值 #FFFFFF
    @param colorSelector {Array} 字体颜色CSS选择器
    @param bgColor {String} 背景颜色 | 默认值 #278cca
    @param bgColorActive {String}  active 背景颜色 | 默认值 是背景色加深20%
    @param bgColorSelector {Array}  背景颜色CSS选择器
    @param borderColor {String}  边框颜色 | 默认值 是背景色加深30%
    @param borderColorSelector {Array}  边框颜色CSS选择器
    @param iconFontColor {String}  字体图标颜色 | 默认值 #333333
    @param iconFontColorSelector {Array}  字体图标颜色CSS选择器
    @example
		ui.Skin.init({
			bgColor : bgColor,
			iconFontColor : bgColor, 
			iconFontColorSelector :[".thumbnail [class^='icon-']",".list-view .icon"]
		});
*/
;ui.Skin = (function(){

	var skinTagId = "BINGOTOUCH-SKIN", TEMPLATE = {
	 	 	BGCOLOR : "@selector { background-color : @bgColor !important;}",
	 	 	BGCOLORACTIVE : "@selector.btn-active { background-color : @bgColorActive !important;}", 
	 	 	COLOR : "@selector { color : @color !important; }",
	 	 	BORDERCOLOR :"@selector {  border-color: @borderColor !important;}"

 		}, 	
 		defaultColorSelect = [ ".header .title", ".footer .navbar [data-role='BTButton']"],
 		defaultBgColorSelect = [ ".header .title",".footer .navbar [data-role='BTButton']", ".footer .sonmenu", ".footer .angle"];

 	var init = function (options){

	 	var settings = {
	 		colorSelector : [],	 	 
	 	 	bgColorSelector : [],
	 	 	borderColorSelector :[],
 	 		iconFontColorSelector :[],
 	 		color : "#FFFFFF",
	 	 	bgColor : "#278cca",
	 	 	bgColorActive : null,
	 	 	borderColor : null,
	 	 	iconFontColor : "#333333",
	 	 	appendElement : "head"
 	 	}

 	 	$.extend(settings,options);
 	 	settings.colorSelector = settings.colorSelector.concat(defaultColorSelect);
 	 	settings.bgColorSelector = settings.bgColorSelector.concat(defaultBgColorSelect);
 	 	settings.borderColorSelector = settings.borderColorSelector.concat(defaultBgColorSelect);
 	 	
 	 	if(!settings.borderColor){
 	 		settings.borderColor = ui.Utils.getDarkColor(settings.bgColor, 0.3); //以背景色加深作为边框颜色
 	 	}

 	 	if(!settings.bgColorActive){
 	 		settings.bgColorActive = ui.Utils.getDarkColor(settings.bgColor, 0.2); 
 	 	}


 	 	$("#" + skinTagId).remove();
 	 	
 	 	var html = "<style id='" + skinTagId + "'>";
	 	 	html += _createCss(settings.colorSelector, "COLOR", { color : settings.color });
	 	 	html += _createCss(settings.iconFontColorSelector, "COLOR", { color : settings.iconFontColor });
	 	 	html += _createCss(settings.bgColorSelector, "BGCOLOR",{ bgColor:settings.bgColor });
	 	 	html += _createCss(settings.bgColorSelector, "BGCOLORACTIVE",{  bgColorActive :settings.bgColorActive });
	 	 	html += _createCss(settings.borderColorSelector, "BORDERCOLOR",{ borderColor :settings.borderColor });
	 	 	html += "</style>";
 	 	$(settings.appendElement).append(html);
 	}

 	var _createCss = function(changeSelectors, templateType, replaceAttrs){
 		var html = "";
   		$.each(changeSelectors ,function(i, selector) {  
   			var res = TEMPLATE[templateType].replace("@selector", selector);
   			for(var attr in replaceAttrs){
   				res = res.replace("@" + attr, replaceAttrs[attr]);
   			}
         	html += res;
        });		
   		return html;
 	}


 	return { 
 		init : init
 	}

})();
 