/**
滑动列表
 @class ui.SwipeListview
*/
/**
    滑动列表初始化
    @method  ui.SwipeListview.init(selector, buttons)
    @param selector {selector|zepto} (必选) 滑动列表选择器
    @param buttons  {JSONObject} 参数,具体参数如下<br/>
    <p> leftBtn  {Array} 左按钮， 具体参数如下</p>
    <ul>
    <li>color ： 背景色</li>
    <li>title ： 显示名称</li>
    <li>width ： 宽度 默认100</li>
	<li>onTap ： 点击触发事件</li>
	</ul>
    <p> rightBtn  {Array} 右按钮 ,具体参数如左按钮</p>
    @example
		ui.SwipeListview.init("#swipe_listview_section .list-view",{
		    leftBtn:[{color:'#8B0000',title:'左按钮1', width:'120', onTap: function(liDom){
		                    alert("左1")
		            }}],
		    rightBtn:[{color:'#8B0000',title:'右按钮1', width:'100', onTap : function(liDom){
		     
		                    alert("右按钮1")
		                    //$(liDom).parent().remove();
		              }},
		              {color:'#228B22',title:'右按钮2', width:'100', onTap : function(liDom){
		 
		                    alert("右按钮2")
		              }}]
		});
*/
;ui.SwipeListview = (function(){

	var defaults = {
		duration :100,
		leftBtn : [],
    	rightBtn : []
	} 

	var init = function(listSelector, options ){
		var $listSelector = $(listSelector);
		if($listSelector.length == 0) return;

		var settings = $.extend({}, defaults, options);

		$listSelector.closest("ul").addClass("swipeListview");

		//列表点击
		$listSelector.delegate("li", "tap", function(e){
 			hide(this, settings.duration);
		});	

		//左右滑动
		var rightBtnWidth =  -1 *  __caculateWidth(settings.rightBtn) + "px";
		$listSelector.delegate("li", "swipeLeft", function(e){	 
			addSwipeBtn(this, settings);
 		 	swipeListAnimation(this, 'left', rightBtnWidth, settings.duration);
 		 	e.preventDefault();
		});	

		var leftBtnWidth =  __caculateWidth(settings.leftBtn) + "px";
		$listSelector.delegate("li", "swipeRight", function(e){
 			addSwipeBtn(this, settings);
	 		swipeListAnimation(this, 'right', leftBtnWidth, settings.duration);
	 		e.preventDefault();
		});	

		//按钮事件
        $listSelector.delegate(".swipe>span",'tap',function(e){
            var isLeft = $(this).parent().hasClass("swipeLeft"),
                index = $(this).attr("i"),
                liObj = $(this).parent().siblings("[data-role='BTButton']")[0];
            if(isLeft){
            	settings.leftBtn[index].onTap(liObj);
            } else{
            	settings.rightBtn[index].onTap(liObj);
            }

            return false;
    	});
	}

	/**
	    打开滑块
	    @method  ui.SwipeListview.show(selector, direct)
	    @param selector {selector|zepto} (必选) 打开滑块DOM对象(可整个list 或 单个li)
	    @param direct  {String} 方向 | 取值为  'Right' 或 'Left'   
	    @example
			ui.SwipeListview.show(".list-view li")  // 打开所有滑块
			ui.SwipeListview.show(".list-view li:eq(0)") //打开单个滑块
	*/
	var show = function(obj, direct ){
		$(obj).trigger("swipe" + (direct || 'Left') );
	}

 	/**
	    关闭滑块
	    @method  ui.SwipeListview.hide(selector, duration)
	    @param selector {selector|zepto} (必选) 关闭滑块DOM对象   (可整个list 或 单个li)
	    @param duration  {Number} 动画时间 | 默认100ms
	    @example
			ui.SwipeListview.hide(".list-view")  // 关闭所有滑块
			ui.SwipeListview.hide(".list-view li:eq(0)") //关闭单个滑块
	*/
	var hide = function(obj , duration){
		duration = duration || defaults.duration;
	 	ui.anim($(obj).find(".swipeRight"),{translateX : "100%"}, duration);
	    ui.anim($(obj).find("[data-role='BTButton']"),{translateX : '0px'}, duration);
	 	ui.anim($(obj).find(".swipeLeft"),{translateX : "-100%"}, duration);
	}

	/*
	 * 添加按钮
	 * @liObj 滑动列
	 */
	function addSwipeBtn(liObj, settings){
	 	var $li = $(liObj), height = $li.height();

		if( settings.rightBtn.length > 0 && $li.find(".swipeRight").length == 0 ){
			var html =  __createBtn(settings.rightBtn, 'swipeRight', height);
	 	   	$li.append(html);
		}

		if(settings.leftBtn.length > 0 && $li.find(".swipeLeft").length == 0 ){
			var html =   __createBtn(settings.leftBtn, 'swipeLeft', height);  
	 	   $li.prepend(html);
		}
	}

	function __createBtn(btns, swipeClass, height){
		var html = '<div class="swipe ' + swipeClass + '">';
  		$.each(btns ,function(i, btn) {  
         	html += '<span i="'+i+'" style="background:'+ btn.color +';width:' + (btn.width || 100) + 'px;height:' + height + 'px;">' + btn.title + '</span>';
        });	
      	html += "</div>";
        return html;
	}

	function __caculateWidth(btns){
		var width = 0;
		$.each(btns ,function(i, btn) {  
         	width += (btn.width || 100) * 1  ;
        });	
        return width;
	}

	/*
	 * 滑动切换
	 * @liObj 滑动列
	 * @direct 滑动方向
	 * zepto的anim对写法有要求,所以下面的anim是有顺序的
	 *（例如有左右两个滑块，如果两个滑块同时向右，那么必须左的那个先anim，不然在低端的android会有bug）
	 */
	function swipeListAnimation(liObj, direct, translateX, duration){
		var liStatus =  __getLiStatus(liObj);
		switch(liStatus){
			case 'rightBtnShow' :
				//close rightBtn
				if(direct == 'right'){	        
           		    ui.anim($(liObj).find("[data-role='BTButton']"),{translateX : '0'}, duration);
           		    ui.anim($(liObj).find(".swipeRight"),{translateX : "100%"}, duration);
				}
			break;
			case 'normal':
				if(direct == 'right'){ //open leftBtn
					ui.anim($(liObj).find(".swipeLeft"),{translateX : '0%' }, duration);
         	 	    ui.anim($(liObj).find("[data-role='BTButton']"),{translateX : translateX}, duration);	 		
				} else{ //open rightBtn			
          	  	    ui.anim($(liObj).find("[data-role='BTButton']"),{translateX : translateX}, duration); 	
          	  	    ui.anim($(liObj).find(".swipeRight"),{translateX : '0%' }, duration);	 
				}
			break;
			case 'leftBtnShow':
				//close leftBtn
				if(direct == 'left'){
			        ui.anim($(liObj).find(".swipeLeft"),{translateX : "-100%"}, duration);
           		    ui.anim($(liObj).find("[data-role='BTButton']"),{translateX : '0'}, duration);
				}
			break;
		}
	}

	/*
	 * 滑动列状态
	 * @liObj 滑动列
	 */	
 	function __getLiStatus(liObj){
		var left = $(liObj).find("[data-role='BTButton']").offset().left;

		if(left < 0 ) {
			return  'rightBtnShow';
		} else if( left > 40 ){ //寫40并沒有多大的關係，因為listview應該要比他長。
			return  'leftBtnShow';
		} else{
			return  'normal';
		}
 	}


	return {
		init : init,
		show : show ,
		hide : hide  //可关闭一个或多个li
	}

})();
 