/**
 * 校验组件
 * @class ui.validation
 */
 /**
    初始化， 该组件有两种校验，一种自动校验，另一种是单次校验
    @method $(selector).validation(options)
    @param autoValidate {Boolean} (可选) 是否开启自动校验（即实时校验）
    @param promptType  {String} (可选) 错误提示显示效果, 取值是 'bubble'| 'inline',默认是 inline 
	@param isPromptAll {Boolean} (可选) 是否显示全部错误 （只在单次校验时生效）
    @example
		//自动校验
		$("#vdForm").validation({
			autoValidate: true
		});
		
		
		//单次校验
		var res = $("#vdForm").validation();
*/
(function($) {

	$.validationConfig = {
		allRules: {
			// Add your regex rules here, you can take telephone as an example
			"required": {
				"executor": "_required"
			},
			"pattern": {
				"executor": "_customRegex"
			},
			"func": {
				"executor": "_funcCall"
			},
			"length": {
				"executor": "_length"
			},
			"range": {
				"executor": "_range"
			},
			"equalToField": {
				"executor": "_confirm",
				"alertText": "输入值与相关信息不相符"
			},
			"url": {
				"regex": /^(http|https|ftp):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)+)(:(\d+))?\/?/i,
				"executor": "_customRegex",
				"alertText": "网址输入不正确"
			},
			"qq": {
				"regex": /^[1-9][0-9]{4,}$/,
				"executor": "_customRegex",
				"alertText": "QQ号码输入不正确"
			},
			"telephone": {
				"regex": /^(\(0\d{2,3}\)|0\d{2,3}-)?[1-9]\d{6,7}(\-\d{1,4})?$/,
				"executor": "_customRegex",
				"alertText": "电话号码输入不正确"
			},
			"mobile": {
				"regex": /^1[3|5|8]\d{9}$/,
				"executor": "_customRegex",
				"alertText": "手机号码输入不正确"
			},
			"email": {
				"regex": /^[a-zA-Z0-9_\.\-]+\@([a-zA-Z0-9\-]+\.)+[a-zA-Z0-9]{2,4}$/,
				"executor": "_customRegex",
				"alertText": "邮箱地址输入不正确"
			},
			"date": {
				"regex": /^[0-9]{4}\-[0-9]{1,2}\-[0-9]{1,2}$/,
				"executor": "_customRegex",
				"alertText": "日期输入格式不正确（YYYY-MM-DD）"
			},
			"identity": {
				"regex": /^([0-9]{17}[0-9X]{1})|([0-9]{15})$/,
				"executor": "_customRegex",
				"alertText": "身份证输入不正确"
			},
			"money": {
				"regex": /^[0-9]+(.[0-9]{2})?$/,
				"executor": "_customRegex",
				"alertText": "金额格式输入不正确"
			},
			"integer": {
				"regex": /^\d+$/,
				"executor": "_customRegex",
				"alertText": "输入值必须是正整数"
			},
			"double": {
				"regex": /^[0-9]+(.[0-9]+)?$/,
				"executor": "_customRegex",
				"alertText": "输入值必须是数值"
			},
			"digit": {
				"regex": /^[0-9]+$/,
				"executor": "_customRegex",
				"alertText": "只能输入数字"
			},
			"noSpecialCaracters": {
				"regex": /^[0-9a-zA-Z]+$/,
				"executor": "_customRegex",
				"alertText": "不允许输入字母和数字之外的特殊字符"
			},
			"letter": {
				"regex": /^[a-zA-Z]+$/,
				"executor": "_customRegex",
				"alertText": "只允许输入英文"
			},
			"chinese": {
				"regex": /^[\u0391-\uFFE5]+$/,
				"executor": "_customRegex",
				"alertText": "只允许输入中文"
			}
		},
		addRules : function(rules){
			$.extend($.validationConfig.allRules, rules);
		}
	}

	var validtorAttr = '[data-validator]';
	var validtorAttrValue = 'data-validator';
	$.fn.validation = function(options) {
		
		options = options || {};
		if(!options.promptType) options.promptType = 'inline';
		var settings = $.extend({
			validationEventTriggers: 	"focusout", //校验默认触发事件
			isPromptAll:				true,		//是否提示全部的错误， 否则只提示第一个错误， 默认是提示全部
			autoValidate: 				false ,		//自动校验
			buildPrompt: 				$.validatePrompt[options.promptType].build,
			closePrompt: 				$.validatePrompt[options.promptType].close
		}, options);
		
		var isInitClosePromptEvent = $(this).data("isInitClosePromptEvent");
		if(!isInitClosePromptEvent){ //关闭错误提示, 不重复添加
			$(this).data("isInitClosePromptEvent", true);
			$(this).find("input"+validtorAttr+",textarea"+validtorAttr).each(function(){
				$(this).bind("focus" , function(){
					settings.closePrompt(this);
				});
			});	
		}
		
		var isInitAutoValidate = $(this).data("isInitAutoValidate");
		//实时校验
		if(settings.autoValidate && !isInitAutoValidate) {
			$(this).data("isInitAutoValidate", true); //阻止重复添加
			//text、 textarea
			$(this).find("input"+validtorAttr+",textarea"+validtorAttr).each(function(){
				var event = $(this).attr("event") || settings.validationEventTriggers ;
				$(this).bind(event , function(){
					_inlineEvent(this, event);
				});
			}) ;		
			
		 	//btcheck、btradio
		 	$(this).find(validtorAttr+"[data-role=BTCheck],"+ validtorAttr+"[data-role=BTRadio]").each(function(){
				$(this).bind("tap" , function(){
					_inlineEvent(this, "tap");
				});
			}) ;	
			
			//TODO:重复初始化问题
			function _inlineEvent(caller,eventType) {
				$.validation.loadValidation(caller, eventType, settings);
			}
		} else {
			return $.validation.validate(this, settings);
		}

	}


	$.validation = {
		loadValidation : function(caller, eventType, settings) { // GET VALIDATIONS TO BE EXECUTED
			var rules = new Array();
			var getRules = $(caller).attr(validtorAttrValue);
			if (!getRules) return true;
			
			var ruleOptions = getRules.match(/\[.+\]/g); //由于正则表达式贪婪模式影响，规则只能存在一个[],否则解析错误
			if (ruleOptions) {
				$.each(ruleOptions, function(index, value) {
					getRules = getRules.replace(this, ("##" + index));
				});
			}

			getRules = getRules.split(",");
			$.each(getRules, function(index, value) {

				var ruleAndOption = this.split("##");
				
				if (ruleAndOption && ruleAndOption.length == 2) {
					rules.push({
						name : ruleAndOption[0],
						options : ruleOptions[ruleAndOption[1]].replace(/^\[|\]$/g, "").split(",")
					});
				} else {
					rules.push({
						name : ruleAndOption[0],
						options : []
					});
				}
			});

			return $.validation.validateCall(caller, rules, eventType, settings)
		},

		validateCall : function(caller, rules, eventType, settings) { // EXECUTE VALIDATION REQUIRED BY THE USER FOR THIS FIELD
			var promptText = "",
				callerName = $(caller).attr("name"),
				errorInfo = $(caller).attr("errorInfo"),
				isError = false;

			$.each(rules, function(i, v) {
				
				var validator = $.validationConfig.allRules[this.name];
				if (validator) {
					eval(validator.executor + "(caller,this,eventType)");
				}  
				
				if (isError) {
					return false;
				}
			});
 
			if (isError == true) {
				promptText = errorInfo || promptText;
				settings.buildPrompt(caller, promptText);
			} else {
				settings.closePrompt(caller);
			}


			/* VALIDATION FUNCTIONS */
			function _required(caller, rule) { // VALIDATE BLANK FIELD
				
				var callerType  = $(caller).attr("type");
				var tagName 	= $(caller)[0].tagName ;
				var dataRole    = $(caller).attr("data-role");
				
				if (callerType == "text" || callerType == "password" || tagName == "TEXTAREA"|| callerType == "hidden") {
					if (!$.trim($(caller).val())) {
						isError = true;
						promptText += _buildPromptText("该输入项必填", rule.options[0]);
					}
				} else if (dataRole == "BTRadio" || dataRole == "BTCheck") {
					callerName = $(caller).attr("name");

					if ($("[name='" + callerName + "'].BTCheck_ON").size() == 0) {
						isError = true;
						if ($("[name='" + callerName + "']").size() == 1) {
							promptText += _buildPromptText("该选项为必选项", rule.options[0]);
						} else {
							promptText += _buildPromptText("必须选择一个选项", rule.options[0]);
						}
					}
				} 
			}

			function _customRegex(caller, rule) { 
				
				if (_isValueEmpty(caller)) {
                    return false;
                }
				var customRule = rule.name;
				var pattern;
				if (customRule == "pattern") {
					pattern = {regex : rule.options[0], alertText : "输入内容不规范"};
				} else {
					pattern = $.validationConfig.allRules[customRule] 
				}
	
				if( typeof pattern.regex == 'string' ){
					pattern.regex = new RegExp(pattern.regex) ;
				}else{
					pattern.regex = eval(pattern.regex);
				}
				
				if (!pattern.regex.test($.trim($(caller).val()))) {
					isError = true;
					promptText += _buildPromptText(pattern.alertText);
				}
			}

			function _funcCall(caller, rule, eventType) { // VALIDATE CUSTOM FUNCTIONS OUTSIDE OF THE ENGINE SCOPE
				var funce = rule.options[0];

				var fn = window[funce];
				if (typeof(fn) === 'function') {
					var fn_result = fn(caller,eventType);
					if (fn_result.isError) {
						isError = true;
						promptText += _buildPromptText(fn_result.errorInfo);
					}
				}
			}

			function _confirm(caller, rule) { // VALIDATE FIELD MATCH
				var confirmField = rule.options[0];
				
				var conf = $("[name='"+confirmField+"']")[0]||$("#" + confirmField)[0] ;//兼容class写法

				if ($(caller).val() != $(conf).val()) {
					isError = true;
					promptText += _buildPromptText($.validationConfig.allRules[rule.name].alertText, rule.options[1]);
				} else {
                    settings.closePrompt(caller);
                }
			}

		 
			function _length(caller, rule) { // VALIDATE LENGTH
				if (_isValueEmpty(caller)) {
					return false;
				}

				var minL = rule.options[0];
				var maxL = rule.options[1];
				var feildLength = $.trim($(caller).val()).length; //.replace(/[^\x00-\xff]/g,'**') 不区分中文长度
				if (minL && feildLength < minL || maxL && feildLength > maxL) {
					isError = true;  
					var tip = "";
					if(minL) tip = '至少' + minL;
					if(maxL) tip += '最多' + maxL;
					promptText += _buildPromptText("已输入"+Math.ceil(feildLength)+"字符，请输入"+tip+"字符" , rule.options[2]);
				}
			}

			function _range(caller, rule) {
				var min = rule.options[0];
				var max = rule.options[1];

				var callerType = $(caller).attr("type");
				var dataRole    = $(caller).attr("data-role");
				if (dataRole == "BTRadio" || dataRole == "BTCheck") {
					var groupSize = $("[name='" + $(caller).attr("name") + "'].BTCheck_ON").size();
					if (groupSize < min || groupSize > max) {
						isError = true;
						var tip = [];
						if(min) tip.push(min);
						if(max) tip.push(max);
						promptText += _buildPromptText("必须选择" +tip.join("到")+ "个选项", rule.options[2]);
					}
				} else {
					if (_isValueEmpty(caller)) {
						return false;
					}
					var inputValue = parseFloat($.trim($(caller).val())) || 0;
					if (inputValue < min || inputValue > max) {
						isError = true;
						promptText += _buildPromptText("输入的值必须在" + min + "到" + max + "之间", rule.options[2]);
					}
				}
			}

			function _buildPromptText(defaultPT, customPT) {
				return customPT ? customPT : defaultPT;
			}

			function _isValueEmpty(caller) {
				return !($(caller).val() && $.trim($(caller).val()).length > 0);
			}

			return isError;
		},
		validate : function(caller, settings) { // FORM SUBMIT VALIDATION LOOPING INLINE VALIDATION
			
			var isError = false;
            var errorInfo = "";

			$(caller).find(validtorAttr).each(function() {
				var res = $.validation.loadValidation(this, null, settings);
				if(res && errorInfo == "") {
					isError = true;
					errorInfo = $(this).attr("errorInfo");
				}
				return settings.isPromptAll || !res;
			});
  
			return {isError:isError,errorInfo:errorInfo};
		}
	}
	
	
	$.validatePrompt = {
		//内嵌
		inline : {
			build : function(caller, promptText){
			 	var html = '<label class="errorInfo">' + promptText + '</label>';
			 	$(caller).siblings(".errorInfo").remove();
			 	$(caller).parent().append(html);			
			},
			close : function(caller){
				$(caller).siblings(".errorInfo").remove();	
			}			
		},
		//冒泡
		bubble : {
			build : function(caller, promptText){
	 			var html = '<label class="errorInfo bubble"><div class="error-inner">' + promptText + '</div><span class="ag"></span></label>';
	 			var pos = $(caller).parent().offset();
	 			var dataRole  = $(caller).attr("data-role");
	 			
	 			var $html = $(html).css({ 
	 				"max-width": pos.width,
	 				top: dataRole == undefined ? pos.height / 2 : pos.height 
	 			});
			 	$(caller).siblings(".errorInfo").remove();
			 	$html.appendTo($(caller).parent()).css("left", pos.width / 2 - $html.width()/ 2);
			},
			close : function(caller){
			 	$(caller).siblings(".errorInfo").remove();	
			}	
		}
	}

})(Zepto);