/**
简化一些常用方法的写法
 @class ui.alias
*/
;(function($){
    /*
     * alias func
     * 简化一些常用方法的写法
     ** /
    /*
     * 完善zepto的动画函数,让参数变为可选
     */
    ui.anim  =  function(el,animName,duration,ease,callback){
        var d, e,c;
        var len = arguments.length;
        for(var i = 2;i<len;i++){
            var a = arguments[i];
            var t = $.type(a);
            t == 'number'?(d=a):(t=='string'?(e=a):(t=='function')?(c=a):null);
        }
        $(el).animate(animName,d|| 250,e|| 'ease', c);
    }
    /**
        显示loading框（ui.Popup.loading(text, icon)的缩写）
        @method  ui.showMask(text, icon)
        @param text {String} 文本 |  当为null时，不显示文字
        @param icon {String}  加载中字体图标  | 默认值 'icon-spinner5'
        @example
            ui.showMask("加载中...");
    */  
    ui.showMask = function(text, icon){
        ui.Popup.loading(text, icon);
    }

    /**
        关闭loading框（ui.Popup.close(true)的缩写）
        @method  ui.hideMask()
        @example
            ui.hideMask();
    */
    ui.hideMask = function(){
        ui.Popup.close(true);
    }
 
    /**
        alert组件（ui.Popup.alert(title,content)的缩写）
        @method ui.alert(title,content)
        @param title {String} 标题
        @param content {String}  内容
        @example
            ui.Popup.alert('提示','这是一个Alert');
    */
    ui.alert = function(title,content){
        ui.Popup.alert(title,content);
    }

    /**
        confirm 组件（ui.Popup.confirm(title,content)的缩写）
        @method  ui.confirm(title,content,okCall,cancelCall,okBtnName,cancelBtnName)
        @param title {String}  标题
        @param content {String}  内容
        @param okCall {Function} 确定按钮handler
        @param cancelCall {Function} 取消按钮handler
        @param okBtnName {String}  （可选）确认按钮名称 | 默认值 '确定'
        @param cancelBtnName {String}  （可选）取消按钮名称 | 默认值 '取消'
        @example
            ui.confirm('提示','这是一个Confirm！',function(){app.hint('你选择了“确定”')},function(){app.hint('你选择了“取消”')});
    */ 
    ui.confirm = function(title,content,okCall,cancelCall, okBtnName, cancelBtnName){
        ui.Popup.confirm(title,content,okCall,cancelCall, okBtnName, cancelBtnName);
    }
    /**
        弹出窗口（ui.Popup.show(options)的缩写）
        @method  ui.popup(options)
        @param options {JSONObject} (可选) 参数，具体参数如下
        <ul>
        <li>height (Number):  高度</li>
        <li>width (Number):  宽度 </li>
        <li>opacity (Number):  遮罩层透明度 | 默认值 0.3</li>
        <li>html (String):  popup内容 </li>
        <li>pos (String):  位置  | 取值 top|top-second|center|bottom|bottom-second </li>
        <li>clickMask2Close (Boolean):  是否点击外层遮罩关闭popup | 默认值 true </li>
        <li>showCloseBtn (Boolean):  是否显示关闭按钮 | 默认值 true </li>
        <li>animation (Boolean):  是否显示动画 | 默认值 true </li>
        <li>duration (Number):  动画执行时间 | 默认值 200ms </li>     
        </ul>
        @example
            ui.popup({
                html: '<div style="height: 100px;text-align: center;font-size: 20px;font-weight: 600;margin-top: 10px;color:#E74C3C ">随意设计你的弹出框吧</div>',
                pos : 'center'
            })
     */
    ui.popup = function(options){
        ui.Popup.show(options);
    }

    /**
        关闭窗口（ui.Popup.close()的缩写）
        @method  ui.closePopup()
        @example
            ui.closePopup();
    */
    ui.closePopup = function(){
        ui.Popup.close();
    }

    /**
        带箭头的弹出框（ui.Popup.popover(html,pos,arrowDirection,onShow)的缩写）
        @method  ui.popover(html,pos,arrowDirection,onShow)
        @param html {String} 弹出框内容
        @param pos {String}  位置  | 取值 top|top-second|center|bottom|bottom-second
        @param arrowDirection {String}  箭头方向 | 取值 top|bottom
        @param onShow  {Function} 显示之前执行
        @example
            var html = '<ul id="popoverlist" class="list-view"> ' +
                            '<li><div data-role="BTButton">Hello, BingoTouch</div></li>' +
                            '<li><div data-role="BTButton">Good good Study</div></li>' +
                            '<li><div data-role="BTButton">Day day up</div></li>' +
                        '</ul>';    
            ui.popover(html,{
                top:'70px',
                left:'10%',
                right:'10%'
            },'top');

            $("#popoverlist").uiwidget();
    */
    ui.popover = function(html,pos,arrowDirection,onShow){
        ui.Popup.popover(html,pos,arrowDirection,onShow);
    }

    /**
        刷新当前页面（单页模式下使用）
        @method  ui.refresh()
        @example
            ui.refresh();
    */
    ui.refresh = function(){
        ui.Page.refresh();
    }

    /**
        页面跳转（单页模式下使用）
        @method  ui.load()
        @param url {String}  页面路径
        @param params {JSONObject} 页面参数
        @example
            ui.load({
                url : "modules/test.html",
                params : {
                    name : "bt"
                }
            });
    */
    ui.load = function(options){
        ui.Page.load(options);
    }

    /**
        页面后退（单页模式下使用）
        @method  ui.back()
        @param pageNumber {Number}  (可选) 退回页码
        @example
            ui.back(); //返回上一页
            ui.back(0); //返回首页
    */
    ui.back = function(pageNumber){
        ui.Page.back(pageNumber);
    }
    
    
    	
	//tab兼容以前写法
	ui.Tab = function(options){
		$(options.container).tab(options);
	}
})(Zepto);