/**
 tab组件
 @class ui.Tab
*/
/**
    tab初始化 （3.3版本之后声明方式变化）
    @method  $(selector).tab(options)
    @param draggable {Boolean} (可选) 是否可以拖拽
    @param onBeforeToggle {Event}  (可选) 切换之前的触发事件
    @param onAfterToggle {Event}  (可选) 切换之后的触发事件
    @example
		//3.3版本之后的建议写法， 兼容以下两种声明方式
		$("#tab_header_container").tab({
			draggable : true,
			onBeforeToggle:function(){//返回true则允许切换Tab
				return true;
			},
			onAfterToggle:function(t){//只有在onBeforeToggle返回true，这个方法才会在Tab切换后被调用
				console.log(t)
			}
		});
		   
		//3.2版本之前的写法
		ui.Tab({
			container:"#tab_header_container",
			draggable : true,
			onBeforeToggle :function(targetId){
				return true; //返回true则允许切换Tab
			},
			onAfterToggle : function(){ //只有在onBeforeToggle返回true，这个方法才会在Tab切换后被调用
			}
		});
*/
(function($, undefined) {
	
	$.ui.define('tab',{
		_data:{
			index:                  0,
	        springBackDis:          15,
			animationTime:          400,
			draggable:				false, 
			onBeforeToggle: 		function(){ return true; },
			onAfterToggle: 			function(){}
		},
		_setup:function() {
			var me = this,
                root = me.root();
			var firstTabId = root.find('[data-role=BTButton]').attr("data-tab");
		    var tabs = $("#"+firstTabId).closest(".ui-tab"),
		    	items = tabs.children();

		    if(tabs.length == 0 ) alert('初始化tab失败,请查看最新版本示例！');
		    tabs.wrapInner("<div class='ui-tab-wheel'>"); 
			me.data({
				tabs : tabs
			});
		},
		_init:function() {
			var me = this,
                index = me.data('index'),
                root = me.root(),
                o = me._data,
                _eventHandler = $.proxy(me._tabEventHandler, me);
             me._setWidth();
             if(o.draggable) $(o.tabs).on('touchstart touchmove touchend touchcancel webkitTransitionEnd', _eventHandler);
             $(".navbar > ul > li ,.navbar table tr td ", root).unbind('tap').bind('tap', function(e){
             	me._navbarEventHandler.call(this, me);
  	    		e.stopPropagation();
    		 });
    		
			$(window).on("ortchange",function(){ //屏幕旋转时
				if(ui.settings.appType == "single" &&　root.closest("section.active").length==0)
					return;
				me._setWidth();
			});
		},
		 /*
         * 设置轮播条及元素宽度,设置选中dot,设置索引map,加载图片
         */
        _setWidth:function(){
            var me = this,
                o = me._data,
                root = o.tabs,
                width = root.width(),
                height = root.height(),
                items = $('tab', root).toArray(),
                length = items.length,
                wheel = $('.ui-tab-wheel', root).width(width * length)[0],
	            i, j,
                l = length;
     
            for(i = 0; i < length; i++) {
                items[i].style.cssText += 'width:'+ width + 'px;position:absolute;-webkit-transform:translate3d(' + i * width + 'px,0,0);z-index:' + (80 - i);
            }
			
			if(o.index){
				o.wheel.style.webkitTransform = 'translate3d(' + (- o.index * width) + 'px,0,0)';
				o.wheel.style.webkitTransitionDuration = '0ms';
			}
			
			
            me.data({
                root:           root[0],
                wheel:          wheel,
                items:          items,
                length:         length,
                width:          width,
                height:         height,
                wheelX:     	-width * (length - 1)
            });
            return me;
        },
		_navbarEventHandler:function(tme){
			var me = this,
				o = tme._data;
			var tabId = $(me).find('[data-role=BTButton]').attr("data-tab"),
				index = $(me).closest("ul").find("li").index($(me));
				
    		var res = o.onBeforeToggle(tabId);
    		if(res){
				tme._move(index, 0);
    		}
		},
		_tabEventHandler:function(e){
			var me = this;
            switch (e.type) {
                case 'touchmove':
                    me._touchMove(e);
                    break;
                case 'touchstart':
                    me._touchStart(e);
                    break;
                case 'touchcancel':
                case 'touchend':
                    me._touchEnd();
                    break;
                case 'webkitTransitionEnd':
                    me._transitionEnd();
                    break;
            }
		},
		
		 /*
         * touchstart事件
         */
        _touchStart:function(e) {
            var me = this;
            me.data({
                pageX:      e.touches[0].pageX,
                pageY:      e.touches[0].pageY,
                S:          false,      //isScrolling
                T:          false,      //isTested
                X:          0,           //horizontal moved
                isExecBeforeFunc: false,
                beforeFuncResult: false
            });
            me.data('wheel').style.webkitTransitionDuration = '0ms';
        },

        /*
         * touchmove事件
         */
        _touchMove:function(e) {
            var o = this._data,
                X = o.X = e.touches[0].pageX - o.pageX;
            if(!o.T) {
                var index = o.index,
                    length = o.length,
                    S = Math.abs(X) < Math.abs(e.touches[0].pageY - o.pageY);
    
                o.T = true;
                o.S = S;
            }
            if(!o.S) {
                o.stopPropagation && e.stopPropagation();
                e.preventDefault();
                var x = X - o.index * o.width;
                if(x < 0 && x > o.wheelX ){
                	if(!o.isExecBeforeFunc){
                		o.beforeFuncResult = o.onBeforeToggle();
                		o.isExecBeforeFunc = true;
                	}  		
                	if(o.beforeFuncResult)
	                	o.wheel.style.webkitTransform = 'translate3d(' + x + 'px,0,0)';
                }
                	
            }
        },

        /*
         * touchend事件
         */
        _touchEnd:function() {
            var me = this,
                o = me._data;
            if(!o.S) {
                var distance = o.springBackDis,
                stepLength = o.X <= -distance ? Math.ceil(-o.X / o.width) : (o.X > distance) ? -Math.ceil(o.X / o.width) : 0;
                o._stepLength = Math.abs(stepLength);
                o.beforeFuncResult && me._slide(o.index + stepLength);
            }
        }, 
		/*
         * 位置判断
         */
        _slide:function(index) {
            var me = this,
                o = me._data,
                length = o.length,
                end = length;
            if(-1 < index && index < end) {
                me._move(index);
            } else if(index >= end) {
                me._move(end - 1);
                o._direction = -1;
            } else {
                me._move(0);
                o._direction =  1;
            }
            return me;
        },

        _move:function(index, animationTime) {
            var me = this,
            	o = me._data;
            o.index = index;
            me._toggleNavbar(index, function(){
            	if(animationTime === undefined) animationTime = o.animationTime;
           		 o.wheel.style.cssText += '-webkit-transition:' + animationTime + 'ms;-webkit-transform:translate3d(-' + index * o.width + 'px,0,0);';	
            })
        },

        /*
         * 滑动结束
         */
        _transitionEnd:function() {
            var me = this,
                o = me._data;
            //me.trigger('slideend', o.dotIndex[o.index]);
        },
        _toggleNavbar: function(index, callback){
        	
        	var me = this,
        		o = me._data,
        		root = me.root();
        	    $li = root.find("li:eq(" + index + ')'),
        	    tabId = $li.find('[data-role=BTButton]').attr("data-tab");
        	
 
	    	$li.siblings().find('[data-role=BTButton]').removeClass('btn-active');
    		$li.find('[data-role=BTButton]').addClass('btn-active');
	        callback();
	        
            //不显示的tab iscroll不refresh
            var id = $("section.active").attr("id") || "body";
            var iscrolls = GC.get(id, "iScroll");

            for (var i = iscrolls.length - 1; i >= 0; i -= 1) {
    
                //是当前页iscroll
                if($(iscrolls[i].wrapper).parent("#"+ tabId).length > 0){
                    iscrolls[i].animating = false;
                } else {
                    iscrolls[i].animating = true;
                }
            }

			o.onAfterToggle(tabId);
    	 
        }
	});
	
})(Zepto);

